' ========================================================================================
' GdipSetCompositingMode
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example creates a Graphics object and sets its compositing mode to
' CompositingModeSourceOver. The code creates a SolidBrush object based on a color that
' has an alpha component of 128. The code passes the address of that brush to the
' GdipFillRectangle function of the Graphics object to fill a rectangle with a color that
' is a half-and-half blend of the brush color and the background color. Then the code sets
' the compositing mode of the Graphics object to CompositingModeSourceCopy and fills a
' second rectangle with the same brush. In that second rectangle, the brush color is not
' blended with the background color.
' ========================================================================================
SUB GDIP_SetCompositingMode (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pGraphics AS DWORD
   LOCAL pAlphaBrush AS DWORD

   hStatus = GdipCreateFromHDC(hdc, pGraphics)

   ' // Create a SolidBrush object with an alpha-blended color.
   hStatus = GdipCreateSolidFill(GDIP_ARGB(180, 255, 0, 0), pAlphaBrush)

   ' // Set the compositing mode to CompositingModeSourceOver, and fill a rectangle.
   hStatus = GdipSetCompositingMode(pGraphics, %CompositingModeSourceOver)
   hStatus = GdipFillRectangle(pGraphics, pAlphaBrush, 0, 0, 100, 100)

   ' // Set the compositing mode to CompositingModeSourceCopy, and fill a rectangle.
   hStatus = GdipSetCompositingMode(pGraphics, %CompositingModeSourceCopy)
   hStatus = GdipFillRectangle(pGraphics, pAlphaBrush, 100, 0, 100, 100)

   ' // Cleanup
   IF pAlphaBrush THEN GdipDeleteBrush(pAlphaBrush)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hDlg AS DWORD
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   DIALOG NEW 0, "GdipSetCompositingMode", , , 280, 170, %WS_OVERLAPPED OR %WS_THICKFRAME OR %WS_SYSMENU OR _
   %WS_MINIMIZEBOX OR %WS_MAXIMIZEBOX OR %WS_VISIBLE OR %DS_CENTER TO hDlg
   DIALOG SET COLOR hDlg, %BLACK, %WHITE
   DIALOG SHOW MODAL hDlg, CALL DlgProc

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Dialog procedure
' ========================================================================================
CALLBACK FUNCTION DlgProc() AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE CBMSG

      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDCANCEL
               IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL, 0
         END SELECT

      CASE %WM_PAINT
         hDC = BeginPaint(CBHNDL, ps)
         GDIP_SetCompositingMode hDC
         EndPaint(CBHNDL, ps)

   END SELECT

END FUNCTION
' ========================================================================================
